<?php
/**
 * Planejamento\Logic
 * @category Php
 * @package  Planejamento\Logic
 * @author   matthaeuscarvalho <matthaeus.carvalho@teknisa.com>
 * @license  www.teknisa.com Teknisa
 * @link     http://localhost:9090/tecfood
 */
namespace Planejamento\Logic;

use Geral\Util\Ambiente;
use Geral\Repositories\Servico;
use Geral\Util\DateUtil;
use Indeterminado\Entities\Setoprodcardtrans;
use Planejamento\Entities\Paramplaneja;
use Planejamento\Entities\Setotipoprat;
use Planejamento\Entities\Servpadr;
use Planejamento\Entities\Setoprodcard;
use Planejamento\Entities\Tipoprat;
use Geral\Entities\Filial;
use Geral\Entities\SetorCoz;
use Planejamento\Entities\Opcprodrec;
use \Geral\Entities\Produto;
use Setforms2\Shared\Registry;

/**
 * Class Planejamento
 * @category Php
 * @package  Planejamento\Logic
 * @author   matthaeuscarvalho <matthaeus.carvalho@teknisa.com>
 * @license  www.teknisa.com Teknisa
 * @link     http://localhost:9090/tecfood
 */
class Planejamento
{
    const WINDOW_ALTER_AUT = 'ger30001';
    const INSERINDO = 'I';
    const ATUALIZANDO = 'U';

    /**
     * Valida o código do tipo de prato
     *
     * @param String $codigo Código
     * @param String $nrorg  Organização
     *
     * @return bool
     */
    public static function validaCodigoTipoPrato($codigo, $nrorg)
    {
        /** @var $dadosPrato Tipoprat */
        $dadosPrato = Tipoprat::getObject(
            array('cdtipoprat' => $codigo, 'nrorg' => $nrorg)
        );

        if ($dadosPrato instanceof Tipoprat) {
            return false;
        }

        return true;
    }

    /**
     * Valida o delete do grupo de serviço
     *
     * @param String $codigo Código
     * @param String $nrorg  Organização
     *
     * @return bool
     */
    public static function validaDeleteGrupoServico($codigo, $nrorg)
    {
        //$sql = "select * from SERVICO where CDGRUPSERV  = $codigo";
        /** @var \Geral\Repositories\Servico $dadosServico Servico */
        $dadosServico = Servico::getObjectsByWhere(
            "CDGRUPSERV = ? AND NRORG = ?",
            array($codigo, $nrorg)
        );
        if (empty($dadosServico)) {
            return true;
        } else {
            tk_abort(tk_get_message_by_name("msg_Pla01500_erroExcluiGrupoServico"));
        }
    }

    /**
     * Valida o número de serviços selecionados
     *
     * @param String $indicadores Indicadores
     *
     * @return bool
     */
    public static function validaNumeroDeServicosSelecionados($indicadores)
    {
        $cdfilial = Ambiente::getCdFilial();
        $nrorg = Ambiente::getNrorg();

        if ($indicadores === 'S') {
            /** @var $servico \Geral\Entities\Servico */
            $servico = \Geral\Entities\Servico::getObjectsByWhere(
                'NRORG = ? AND CDFILIAL = ? AND IDINDICADOR = ?',
                array(
                    $nrorg,
                    $cdfilial,
                    $indicadores
                )
            );
            $servicos = count($servico);
            if ($servicos > 5) {
                tk_abort(tk_get_message_by_name("msg_pla99100_LimiteExcedidoServico"));
            }
        }

        return true;
    }

    /**
     * Gera lista de pratos
     *
     * @param String $linhas Linhas
     *
     * @return bool|string
     */
    public static function geraListaPratos($linhas)
    {
        
        $listaPrato = '';
        if (empty($linhas) == false) {
            $listaPrato = '_';
            foreach ($linhas as $linha) {
                foreach ($linha as $indice => $cdprato) {
                    if ($indice == 'CDPRATO') {
                        $listaPrato .= $cdprato . '_';
                    }
                }
            }
        } else {
            return false;
        }
        return $listaPrato;
    }

    /**
     * Gera Lista de todos nutrientes
     *
     * @param String $nrorg Organização
     *
     * @return string
     */
    public static function geraListaTodosNutrientes($nrorg)
    {
        /** @var $nutrientes \Geral\Entities\Nutriente[] */
        $nutrientes = \Geral\Entities\Nutriente::getObjectsByWhere('NRORG = ?', array($nrorg));

        $listaNutriente = '';
        foreach ($nutrientes as $nutriente) {
            $listaNutriente .= '_' . $nutriente->getCdnutriente() . '_';
        }

        return $listaNutriente;
    }

    /**
     * Retorna o próximo código do padrão de serviço
     *
     * @param String $nrorg Organização
     *
     * @return null
     */
    public static function retornaProxCdservpadr($nrorg)
    {
        /** @var \Planejamento\Entities\Servpadr $servpadr Padrão de Serviço */
        $servpadr = Servpadr::getObjectByWhere('NRORG = ?', array($nrorg), 'CDSERVPADR DESC');
        if ($servpadr) {
            $max = $servpadr->getCdservpadr();
            return $max+1;
        }
        else {
            return 1;
        }
    }

    /**
     * Atualizar o serviço correspondentes ao Padrão de Serviço informado
     *
     * @param Array  $cdfilial  Unidade
     * @param String $nrorg     Organização
     * @param Array  $rows      Linhas
     *
     * @throws \Exception
     * @return null
     */
    public static function atualizaServico($cdfilial, $nrorg, $rows)
    {
        try {
            $tfilial = '';

            if ($cdfilial == 'A') {
                $pfilial = '';
                $tfilial = 'T';
                foreach ($rows as $lin) {
                    Servico::atualizaPadraoServico(
                        $nrorg, $pfilial, $tfilial, $lin['CDPRODUTO'],
                        $lin['NMSERVPADR'], $lin['CDSERVPADR']
                    );
                }
                tk_message(tk_get_message_by_name('msg_gep_Atualizado'), 'success');
            }
            else {
                //se não selecionar nenhuma filial (unidade), nenhum serviço será atualizado
                if ($cdfilial == 'N') {
                    $pfilial = '';
                    tk_warning('Sem serviços para alterar.');
                } else {
                    //para cada filial (unidade)
                    foreach ($cdfilial as $pfilial) {
                        //realiza o update dos valores de cada linha selecionada
                        self::_atualizaLinhaPadraoServico($nrorg, $rows, $pfilial, $tfilial);
                    }
                    tk_message(tk_get_message_by_name('msg_gep_Atualizado'), 'success');
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Atualiza o serviço correspondentes ao Padrão de Serviço informado da filial informada
     *
     * @param String $nrorg   Organização
     * @param Array  $rows    Linhas
     * @param Array  $pfilial Unidade
     * @param String $tfilial Todas unidades
     *
     * @throws \Exception
     * @return null
     */
    private static function _atualizaLinhaPadraoServico($nrorg, $rows, $pfilial, $tfilial)
    {
        try {
            foreach ($rows as $lin) {
                Servico::atualizaPadraoServico(
                    $nrorg, $pfilial['cdfilial'], $tfilial, $lin['CDPRODUTO'], $lin['NMSERVPADR'], $lin['CDSERVPADR']
                );
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Retorna o próximo código de exceção do setor de cardápio
     *
     * @param String $nrorg      Organização
     * @param String $cdtipoprat Tipo de Prato
     * @param String $cdfilial   Unidade
     * @param String $cdsetorcoz Setor
     * @param String $cdproduto  Produto
     *
     * @return null
     */
    public static function retornaProxNrseqsetoprc(
        $nrorg, $cdtipoprat, $cdfilial, $cdsetorcoz, $cdproduto
    ) {
        /**
         * @var \Planejamento\Entities\Setoprodcard $setoprodcard Setor do cardápio
         */
        $setoprodcard = Setoprodcard::getObjectByWhere(
            'NRORG = ? AND CDTIPOPRAT = ? AND CDFILIAL = ? AND CDSETORCOZ = ? AND
            CDPRODUTO = ?', array($nrorg, $cdtipoprat, $cdfilial, $cdsetorcoz, $cdproduto),
            'NRSEQSETOPRC DESC'
        );
        if ($setoprodcard) {
            $max = $setoprodcard->getNrseqsetoprc() + 1;
            $proximo = \Geral\Util\Util::zeroFill($max, 2);
            return $proximo;
        }
        else return '01';
    }

    /**
     * Atualiza o setor do cardápio padrão
     *
     * @param String $cdfilial       Unidade
     * @param String $cdtipoprat     Tipo de prato
     * @param String $cdsetorcoz     Setor
     * @param String $nrorg          Organização
     * @param String $dtinclusao     Data de inclusão
     * @param String $cdoperinclusao Operador de inclusão
     * @param String $cdoperultatu   Operador de alteração
     * @param String $nrorgultatu    Organização alteração
     * @param String $dtultatu       Data última atualização
     *
     * @throws \Exception
     * @return null
     */
    public static function atualizaSetorCardapioPadrao(
        $cdfilial, $cdtipoprat, $cdsetorcoz, $nrorg, $dtinclusao, $cdoperinclusao,
        $cdoperultatu, $nrorgultatu, $dtultatu
    ) {
        try {
            $connection = \Setforms2\Shared\Registry::getEntityManager();
            $connection->beginTransaction();

            if ($dtinclusao == null) {
                $dtinclusao = DateUtil::getDataAtual();
            }
            if ($cdoperinclusao == null) {
                $cdoperinclusao = Ambiente::getCdOperador();
            }

            /**
             * Setor do Tipo do Prato
             * @var \Planejamento\Entities\Setotipoprat $setotipoprat
             */
            $setotipoprat = Setotipoprat::getObjectByWhere(
                'NRORG = ? AND CDFILIAL = ? AND CDTIPOPRAT = ?',
                array($nrorg, $cdfilial, $cdtipoprat)
            );

            //se não houver registro, será inserção
            if (empty($setotipoprat) == true) {
                //define o objeto para inserção
                $setotipoprat = new \Planejamento\Entities\Setotipoprat();
            }

            //Captura uma entidade do setor
            /**
             * Setor
             * @var \Geral\Entities\Setorcoz $setorcoz
             */
            $setorcoz = Setorcoz::getObjectByWhere(
                'NRORG = ? AND CDFILIAL = ? AND CDSETORCOZ = ?',
                array($nrorg, $cdfilial, $cdsetorcoz)
            );

            $setotipoprat->setCdfilial($cdfilial);
            $setotipoprat->setCdtipoprat($cdtipoprat);
            $setotipoprat->setSetorcoz($setorcoz);
            $setotipoprat->setNrorg($nrorg);
            $setotipoprat->setDtinclusao($dtinclusao);
            $setotipoprat->setDtultatu($dtultatu);
            $setotipoprat->setNrorginclusao($nrorg);
            $setotipoprat->setCdoperinclusao($cdoperinclusao);
            $setotipoprat->setNrorgultatu($nrorgultatu);
            $setotipoprat->setCdoperultatu($cdoperultatu);

            $setotipoprat->persist();

            $connection->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

    /**
     * Retorna o próximo código de exceções (transportada)
     *
     * @param String $nrorg    Organização
     * @param String $cdfilial Unidade
     *
     * @return null
     */
    public static function retornaProxNrseqsetotrans($nrorg, $cdfilial)
    {
        /**
         * Setor do cardápio
         * @var \Indeterminado\Entities\Setoprodcardtrans $setoprodcardtrans
         */
        $setoprodcardtrans = Setoprodcardtrans::getObjectByWhere(
            'NRORG = ? AND CDFILIAL
            = ?', array($nrorg, $cdfilial), 'NRSEQSETOTRANS DESC'
        );
        if ($setoprodcardtrans) {
            $max = $setoprodcardtrans->getNrseqsetotrans();
            return $max+1;
        }
        else return 1;
    }

    /**
     * @param $cdarvprod
     * @param $nmproduto
     * @param $cdprorigrec
     * @param $nrorg
     *
     * @return null
     * @throws \Exception
     */
    public static function atualizaProdutoPrincipalReceita($cdarvprod, $nmproduto, $cdprorigrec, $nrorg)
    {
        try
        {
            $connection = \Setforms2\Shared\Registry::getEntityManager();
            $connection->beginTransaction();

             /** @var \Planejamento\Entities\Opcprodrec $opcprodrec  */
            $opcprodrec = Opcprodrec::getObjectByWhere(
                'NRORG = ? AND CDPRORIGREC = ? AND CDPROPCREC = ?',
                array($nrorg, $cdprorigrec, $cdprorigrec)
            );

            //se a consulta for vazia
            if ($opcprodrec == false){
                //define o objeto para inserção
                $opcprodrec = new \Planejamento\Entities\Opcprodrec();
            }

            $opcprodrec->setCdprorigrec($cdprorigrec);
            $opcprodrec->setCdpropcrec($cdprorigrec);
            $opcprodrec->setNrorg($nrorg);
            $opcprodrec->setNrorginclusao(Ambiente::getNrorg());
            $opcprodrec->setCdoperinclusao(Ambiente::getCdOperador());
            $opcprodrec->setNrorgultatu(Ambiente::getNrorg());
            $opcprodrec->setCdoperultatu(Ambiente::getCdOperador());
            $opcprodrec->setDtinclusao(DateUtil::getDataAtual());
            $opcprodrec->setDtultatu(DateUtil::getDataAtual());

            $opcprodrec->persist();
            $connection->flush();
            $connection->commit();
            tk_message(tk_get_message_by_name('msg_gep_Atualizado'), 'success');
            tk_reload_data_with_filter('tab_pla06500_001');
            tk_reload_data_with_filter('tab_pla06500_002');
        } catch (\Exception $e) {
            
            throw $e;
        }
    }

    /**
     * @param $cdpropcrec
     * @param $nrorg
     * @param $cdprorigrec
     *
     * @return null
     * @throws \Exception
     */
    public static function atualizaProdutoOpcionalReceita($cdpropcrec, $nrorg, $cdprorigrec) {
        try
        {
            $connection = \Setforms2\Shared\Registry::getEntityManager();
            $connection->beginTransaction();
            
             /** @var \Planejamento\Entities\Opcprodrec $opcprodrec  */
            $opcprodrec = Opcprodrec::getObjectByWhere(
                'NRORG = ? AND CDPRORIGREC = ? AND CDPROPCREC = ?',
                array($nrorg, $cdprorigrec, $cdpropcrec)
            );

            //se a consulta for vazia
            if ($opcprodrec == false) {
                //define o objeto para inserção
                $opcprodrec = new \Planejamento\Entities\Opcprodrec();
            }            
            
            $opcprodrec->setCdpropcrec($cdpropcrec);
            $opcprodrec->setCdprorigrec($cdprorigrec);
            $opcprodrec->setNrorg($nrorg);
            $opcprodrec->setNrorginclusao(Ambiente::getNrorg());
            $opcprodrec->setCdoperinclusao(Ambiente::getCdOperador());
            $opcprodrec->setNrorgultatu(Ambiente::getNrorg());
            $opcprodrec->setCdoperultatu(Ambiente::getCdOperador());
            $opcprodrec->setDtinclusao(DateUtil::getDataAtual());
            $opcprodrec->setDtultatu(DateUtil::getDataAtual());

            $opcprodrec->persist();

            $connection->flush();
            $connection->commit();
            tk_message(tk_get_message_by_name('msg_gep_Atualizado'), 'success');
            tk_reload_data_with_filter();        

        } catch (\Exception $e) {            
            throw $e;
        }
    }

    /**
     * @param $cdprorigrec
     * @param $nrorg
     *
     * @return null
     * @throws \Exception
     */
    public static function excluiProdutoPrincipal($cdprorigrec, $nrorg)
    {
        try {
            $connection = \Setforms2\Shared\Registry::getEntityManager();
            $connection->beginTransaction();
            /** @var \Planejamento\Entities\Opcprodrec $opcprodrec */
            $opcprodrec = Opcprodrec::getObjectsByWhere(
                'NRORG = ? AND CDPRORIGREC = ?',
                array($nrorg, $cdprorigrec)
            );
            foreach ($opcprodrec as $produto){
                $produto->delete();
            }
            $connection->flush();
            $connection->commit();
            tk_message(tk_get_message_by_name('msg_excluidoSucesso'), 'success');
            tk_reload_data_with_filter();
//            \Setforms2\Shared\Registry::getEntityManager()->flush();

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $cdpropcrec
     * @param $nrorg
     * @param $cdprorigrec
     *
     * @return null
     * @throws \Exception
     */
    public static function excluiProdutoOpcional($cdpropcrec, $nrorg, $cdprorigrec)
    {
        try {
            $connection = \Setforms2\Shared\Registry::getEntityManager();
            $connection->beginTransaction();
            /** @var \Planejamento\Entities\Opcprodrec $opcprodrec */
            $opcprodrec = Opcprodrec::getObjectByWhere(
                'NRORG = ? AND CDPRORIGREC = ? AND CDPROPCREC = ?',
                array($nrorg, $cdprorigrec, $cdpropcrec)
            );

            $opcprodrec->delete();
            $connection->flush();
            $connection->commit();
            tk_message(tk_get_message_by_name('msg_excluidoSucesso'), 'success');
            tk_reload_data_with_filter();

        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * @param $cdarvprod
     * @param $cdprorigrec
     * @return null
     * @throws \Exception
     */
    public static function validaProdutoJaCadastrado($cdarvprod, $cdprorigrec){
        /** @var $produtoRepository \Geral\Repositories\Produto */
        //$produtoRepository = \Geral\Repositories\Produto::getRepository();
        $produtoRepository = Produto::getRepository();
        $cdarvprinc = $produtoRepository->validaProdutoJaCadastrado($cdarvprod);

        if (empty($cdarvprinc) == false){
            //Produto Principal já cadastrado.
            throw new \Exception(tk_get_message_by_name('msg_pla06500_ProdPrincJaCad'));
        }
        else{
            $cdarvopc = $produtoRepository->validaProdutoOpcionalJaCadastrado($cdarvprod);

            foreach($cdarvopc as $cdarvore){
                if ($cdarvore['CDPRORIGREC'] == $cdprorigrec){
                    //Produto Opcional já cadastrado.
                    throw new \Exception(tk_get_message_by_name('msg_pla06500_ProdOpcJaCad'));
                }
            }
        }

    }


    /**
     * @param $cdarvprod
     * @param $nrorg
     *
     * @return null
     * @throws \Exception
     */
    public static function validaProdutoCompra($cdarvprod, $nrorg){
        /** @var \Geral\Entities\Produto $produto  */
        $produto = \Geral\Entities\Produto::getObjectByWhere(
            'CDARVPROD = ? AND NRORG = ?',
            array($cdarvprod, $nrorg)
        );
        if (($produto->getCdprodesto() != null) and (
            $produto->getCdprodesto() != $produto->getCdproduto())
        ) {
            //Tabela não pode conter produto de compra.
            throw new \Exception(tk_get_message_by_name('msg_pla06500_prodcompra'));
        }
    }

    /**
     * Salva a Exceção do Setor do Cardápio
     *
     * @param String $nrseqsetoprc   Código
     * @param String $cdproduto      Produto
     * @param String $cdtipoprat     Tipo de prato
     * @param String $cddietpad      Dieta
     * @param String $cdsetorcoz     Setor
     * @param String $cdfilial       Unidade
     * @param String $nrorg          Organização
     * @param String $dtinclusao     Data de inclusão
     * @param String $dtultatu       Data de atualização
     * @param String $nrorginclusao  Organização inclusão
     * @param String $cdoperinclusao Operador de inclusão
     * @param String $nrorgultatu    Organização atualização
     * @param String $cdoperultatu   Operador de atualização
     * @param String $estado         Estado
     *
     * @throws \Exception
     * @return null
     */
    public static function salvaExcecaoCardapio(
        $nrseqsetoprc, $cdproduto, $cdtipoprat, $cddietpad, $cdsetorcoz,
        $cdfilial, $nrorg, $dtinclusao, $dtultatu, $nrorginclusao,
        $cdoperinclusao, $nrorgultatu, $cdoperultatu, $estado
    ) {
        try {
            $connection = \Setforms2\Shared\Registry::getEntityManager();
            $connection->beginTransaction();

            $dtultatu = DateUtil::getDataDeString($dtultatu);
            $dtinclusao = DateUtil::getDataDeString($dtinclusao);

            if ($estado == self::INSERINDO) {
                $existe = Setoprodcard::getObjectByWhere(
                    'NRORG = ? AND CDTIPOPRAT = ? AND CDFILIAL = ? AND CDPRODUTO = ? AND CDDIETPAD = ?',
                    array($nrorg, $cdtipoprat, $cdfilial, $cdproduto, $cddietpad)
                );
                if ($existe) {
                    throw new \Exception(tk_get_message_by_name('msg_excecao_cadastrada'));
                }
                /** @var $obj \Planejamento\Entities\Setoprodcard */
                $excecaoCardapio = new \Planejamento\Entities\Setoprodcard();
                $nrseqsetoprc = Planejamento::retornaProxNrseqsetoprc(
                    $nrorg, $cdtipoprat, $cdfilial, $cdsetorcoz, $cdproduto
                );
            }
            elseif ($estado == self::ATUALIZANDO) {
                /** @var $obj \Planejamento\Entities\Setoprodcard */
                $excecaoCardapio = Setoprodcard::getObjectByWhere(
                    'NRORG = ? AND CDTIPOPRAT = ? AND CDFILIAL = ? AND CDSETORCOZ
                    = ? AND CDPRODUTO = ? AND NRSEQSETOPRC = ?',
                    array($nrorg, $cdtipoprat, $cdfilial, $cdsetorcoz, $cdproduto,
                        $nrseqsetoprc)
                );
            }

            $excecaoCardapio->setNrseqsetoprc($nrseqsetoprc);
            $excecaoCardapio->setCdproduto($cdproduto);
            $excecaoCardapio->setCdtipoprat($cdtipoprat);
            $excecaoCardapio->setCddietpad($cddietpad);
            $excecaoCardapio->setCdsetorcoz($cdsetorcoz);
            $excecaoCardapio->setCdfilial($cdfilial);
            $excecaoCardapio->setNrorg($nrorg);
            $excecaoCardapio->setDtinclusao($dtinclusao);
            $excecaoCardapio->setDtultatu($dtultatu);
            $excecaoCardapio->setNrorginclusao($nrorginclusao);
            $excecaoCardapio->setCdoperinclusao($cdoperinclusao);
            $excecaoCardapio->setNrorgultatu($nrorgultatu);
            $excecaoCardapio->setCdoperultatu($cdoperultatu);
            $excecaoCardapio->setIdativo('S');

            $excecaoCardapio->persist();
            $connection->flush();
            $connection->commit();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function atualizaCamposSetores ($nrorg, $cdparplaneja, $cdfilial, $cdsetorfixo, $cdsetorpercapita,  $data, $cdoperinclusao)
    {
        /** @var $registro Paramplaneja */
        $registro = Paramplaneja::getObjectByWhere('NRORG = ? AND CDPARPLANEJA = ? AND CDFILIAL = ?' ,
            array($nrorg, $cdparplaneja, $cdfilial));

        try {
            $conn = Registry::getEntityManager();
            $conn->beginTransaction();
            $registro->updateSetores($nrorg, $cdsetorfixo, $cdsetorpercapita, $data, $cdoperinclusao);
            $conn->flush();
            $conn->commit();
            return true;
        } catch (\Exception $e) {
            $conn->rollback();
            tk_message($e->getMessage(), 'error');
            return false;
        }
    }

}